import torch


def multi_modal_causal_self_attention_mask(x):
    """
    Applies a multi-modal causal self-attention mask. This mask allows query tokens to attend to all
    other query tokens and text tokens to attend only to preceding text tokens and all query tokens.

    Args:
    - x (torch.Tensor): the input tensor of shape [batch_size, seqlen, dim]

    Returns:
    - torch.Tensor: the mask tensor of shape [batch_size, seqlen, seqlen] with 0s where attention is allowed
                    and float('-inf') where it is not, suitable for adding to the raw attention scores.
    """
    batch_size, seqlen, _ = x.shape
    # Initialize mask to all ones
    mask = torch.ones((seqlen, seqlen), dtype=torch.float32)
    # Create a causal mask for the text tokens
    causal_mask = torch.tril(
        torch.ones((seqlen // 2, seqlen // 2), dtype=torch.float32)
    )
    mask[-(seqlen // 2) :, -(seqlen // 2) :] = causal_mask
    # Invert the mask so that 0s are where attention is allowed and float('-inf') where it is not
    mask = torch.log(mask)

    # Expand the mask for the batch size
    mask = mask.repeat(batch_size, 1, 1)

    return mask


batch_size = 2
seqlen = 8
dim = 512


# Example to test the function with dummy data
x_dummy = torch.rand(batch_size, seqlen, dim)  # Dummy data
multi_modal_causal_mask = multi_modal_causal_self_attention_mask(
    x_dummy
)
print(multi_modal_causal_mask.shape)



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['qformer']

package_data = \
{'': ['*']}

install_requires = \
['einops', 'swarms', 'torch', 'zetascale']

setup_kwargs = {
    'name': 'qformer',
    'version': '0.0.3',
    'description': 'qformer - Pytorch',
    'long_description': '[![Multi-Modality](agorabanner.png)](https://discord.gg/qUtxnK2NMf)\n\n\n# Qformer\nImplementation of Qformer from BLIP2 in Zeta Lego blocks. The implementation is here straight from Figure 2. In particular the image block and text block.\n\n## Install\n`pip3 install qformer`\n\n\n## Usage\n```python\nimport torch\nfrom qformer import QFormer\n\nx = torch.randn(1, 32, 512)\nimg = torch.randn(1, 32, 512)\n\nqformer = QFormer(512, 8, 8, 0.1, 2, 2)\ny = qformer(x, img)\nprint(y.shape)\n```\n\n\n# License\nMIT\n\n\n\n',
    'author': 'Kye Gomez',
    'author_email': 'kye@apac.ai',
    'maintainer': 'None',
    'maintainer_email': 'None',
    'url': 'https://github.com/kyegomez/qformer',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.6,<4.0',
}


setup(**setup_kwargs)
